<!--
  - File: tabulate.xsl
  -
  - Convert an XML file to a table.
  -
  - The input document is the data to be converted.
  - The output document is in a table format
  - The converstion is driven by a tableSpecification file passed in a parameter.
  -
  - This uses EXSLT extensions (to process the paths).
  -->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
  xmlns:dyn="http://exslt.org/dynamic"
  extension-element-prefixes="dyn"
  exclude-result-prefixes="dyn" >

<xsl:param name="tableSpecificationFileName"/>

<xsl:variable name="tableSpecification" select="document($tableSpecificationFileName)"/>
<xsl:variable name="extractRoot" select="/*"/>

<xsl:template match="/*">

  <xsl:if test="not($tableSpecification)">
    <xsl:message terminate="yes">Can not read table specification from <xsl:value-of select="$tableSpecificationFileName"/></xsl:message>
  </xsl:if>

  <TableExport>
    <xsl:apply-templates select="$tableSpecification/TableExport/TableList"/>
  </TableExport>

</xsl:template>

<xsl:template match="TableList">
  <TableList>
    <xsl:apply-templates select="Table"/>
  </TableList>
</xsl:template>

<xsl:template match="Table">
  <xsl:variable name="columnList" select="ColumnList"/>
  <Table>
    <xsl:copy-of select="tableName"/>
    <xsl:apply-templates select="ColumnList" mode="columnSpecification"/>
    <RowList>
      <xsl:variable name="rowPath" select="rowPath"/>
      <!-- $extractRoot is a single node, so we only iterate once -->
      <xsl:for-each select="$extractRoot">
        <xsl:for-each select="dyn:evaluate($rowPath)">
          <xsl:variable name="row" select="."/>
          <Row>
            <FieldList>
              <xsl:for-each select="$columnList/Column">
                <xsl:variable name="column" select="."/>
                <!-- $row is a single node, so we only iterate once -->
                <xsl:for-each select="$row">
                  <!-- We do this so that field is always a value, not a node set -->
                  <xsl:variable name="field">
                    <xsl:value-of select="dyn:evaluate($column/fieldPath)"/>
                  </xsl:variable>
                  <!-- We don't know the type of field, so we check that there is something there -->
                  <xsl:if test="$field != ''">
                    <Field>
                      <columnName><xsl:value-of select="$column/columnName"/></columnName>
                      <value><xsl:value-of select="$field"/></value>
                    </Field>
                  </xsl:if>
                </xsl:for-each>
              </xsl:for-each>
            </FieldList>
          </Row>
        </xsl:for-each>
      </xsl:for-each>
    </RowList>
  </Table>
</xsl:template>

<xsl:template match="ColumnList" mode="columnSpecification">
  <ColumnList>
    <xsl:apply-templates select="Column" mode="columnSpecification"/>
  </ColumnList>
</xsl:template>

<xsl:template match="Column" mode="columnSpecification">
  <Column>
    <xsl:copy-of select="columnName"/>
    <xsl:copy-of select="dataType"/>
  </Column>
</xsl:template>

</xsl:stylesheet>



