<!--
  - File: tabulated_to_html.xsl
  -
  - Create a simple web page showing tabular data.
  -->
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:java="http://xml.apache.org/xslt/java"
  extension-element-prefixes="java" version="1.0">

<xsl:output method="xml" omit-xml-declaration="yes" encoding="ASCII"/>

<xsl:template match="TableExport">

  <html>
    <head>
      <title>Table Export</title>
      <style type="text/css">
        body {
          font-family: Arial, Helvetica, Sans-Serif;
          font-size: 80%;
        }
        table {
          border-spacing: 0px;
          border-collapse: collapse;
        }
        table tr td,
        table tr th {
          font-family: Arial, Helvetica, Sans-Serif;
          font-size: 80%;
          border: 1px solid black;
          vertical-align: top;
          text-align: left;
          padding: 2px;
        }
        table tr td.text {
          white-space: pre;
        }
      </style>
    </head>
    <body>
      <h1>Table Export</h1>
      <xsl:apply-templates select="TableList/Table"/>
    </body>
  </html>

</xsl:template>

<xsl:template match="Table">
  <h2><xsl:value-of select="tableName"/></h2>
  <table>
    <tr>
      <xsl:apply-templates select="ColumnList/Column"/>
    </tr>
    <xsl:apply-templates select="RowList/Row"/>
  </table>
</xsl:template>

<xsl:template match="Column">
  <th class="{dataType}">
    <xsl:value-of select="columnName"/>
  </th>
</xsl:template>

<!--
  - Data row
  - To cope with mis-ordered fields and null data, we iterate over the columns
  - and output the matching fields.
  -->
<xsl:template match="Row">
  <xsl:variable name="row" select="."/>
  <tr>
    <xsl:for-each select="../../ColumnList/Column">
      <xsl:variable name="columnName" select="columnName"/>
      <xsl:variable name="dataType" select="dataType"/>
      <xsl:variable name="field" select="$row/FieldList/Field[columnName = $columnName]"/>
      <td class="{$dataType}">
        <xsl:choose>
          <xsl:when test="not($field)">
            <xsl:text>&#160;</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$field/value"/>
          </xsl:otherwise>
        </xsl:choose>
      </td>
    </xsl:for-each>
  </tr>
</xsl:template>

</xsl:stylesheet>
