<!--
  - Create a directory structure from a single XML file.
  - File is format
  - <root>
  -   <directory name="xxx">
  -     other directory and file elements
  -   </directory>
  -   <file name="xxx">
  -     xml contents of file
  -   </file>
  - </root>
  - Important:
  - <file> must contain a single XML element
  -
  - Parameter outputDirectory specifies output directory.
  - should not end in a slash.
  -->
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
  xmlns:redirect="http://xml.apache.org/xalan/redirect"
  extension-element-prefixes="redirect">

<xsl:param name="outputDirectory" select="''"/>

<xsl:variable name="outputDirectoryToUse">
  <xsl:choose>
    <xsl:when test="$outputDirectory = ''">
      <xsl:value-of select="/directory/@name"/>
      <xsl:text>/</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$outputDirectory"/>
      <xsl:text>/</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<xsl:template match="/*">
  <FileList>
    <xsl:apply-templates select="*"/>
  </FileList>
</xsl:template>

<xsl:template match="directory">
  <xsl:apply-templates select="*"/>
</xsl:template>

<xsl:template match="file">
  <xsl:variable name="filePath">
    <xsl:call-template name="BuildPath"/>
  </xsl:variable>
  <redirect:write file="{$filePath}">
    <xsl:copy-of select="*"/>
  </redirect:write>
  <file><xsl:value-of select="$filePath"/></file>
</xsl:template>

<xsl:template name="BuildPath">
  <xsl:choose>
    <xsl:when test="count(ancestor::*) &gt; 1">
      <xsl:for-each select="..">
        <xsl:call-template name="BuildPath"/>
        <xsl:text>/</xsl:text>
      </xsl:for-each>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$outputDirectoryToUse"/>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:value-of select="@name"/>
</xsl:template>


</xsl:stylesheet>
